@@ -16,6 +16,7 @@ import uuid
 
 import requests
 
+
 # import request objects from various frameworks, if available
 try:
     from webob import BaseRequest as WebobBaseRequest
@@ -47,6 +48,51 @@ try:
 except ImportError:
     BottleRequest = None
 
+
+def get_request():
+    """
+    Get the current request object. Implementation varies on
+    library support. Modified below when we know which framework
+    is being used.
+    """
+
+    # TODO(cory): add in a generic _get_locals_request() which
+    # will iterate up through the call stack and look for a variable
+    # that appears to be valid request object.
+    for fn in (_get_bottle_request,
+               _get_flask_request,
+               _get_pyramid_request,
+               _get_pylons_request):
+        try:
+            req = fn()
+            if req is not None:
+                return req
+        except:
+            pass
+
+    return None
+
+
+def _get_bottle_request():
+    from bottle import request
+    return request
+
+
+def _get_flask_request():
+    from flask import request
+    return request
+
+
+def _get_pyramid_request():
+    from pyramid.threadlocal import get_current_request
+    return get_current_request()
+
+
+def _get_pylons_request():
+    from pylons import request
+    return request
+
+
 BASE_DATA_HOOK = None
 
 log = logging.getLogger(__name__)
@@ -340,7 +386,7 @@ def _report_exc_info(exc_info, request, extra_data, payload_data):
     data['server'] = _build_server_data()
 
     if payload_data:
-        data.update(payload_data)
+        data = dict_merge(data, payload_data)
 
     payload = _build_payload(data)
     send_payload(payload)
@@ -372,7 +418,7 @@ def _report_message(message, level, request, extra_data, payload_data):
     data['server'] = _build_server_data()
 
     if payload_data:
-        data.update(payload_data)
+        data = dict_merge(payload_data)
 
     payload = _build_payload(data)
     send_payload(payload)
@@ -507,13 +553,13 @@ def _build_request_data(request):
 
     if WerkzeugLocalProxy and isinstance(request, WerkzeugLocalProxy):
         actual_request = request._get_current_object()
-        return _build_werkzeug_request_data(request)
+        return _build_werkzeug_request_data(actual_request)
 
     # tornado
     if TornadoRequest and isinstance(request, TornadoRequest):
         return _build_tornado_request_data(request)
 
-    #bottle
+    # bottle
     if BottleRequest and isinstance(request, BottleRequest):
         return _build_bottle_request_data(request)
 
@@ -750,6 +796,22 @@ def _django_extract_user_ip(request):
     return request.environ['REMOTE_ADDR']
 
 
+# http://www.xormedia.com/recursively-merge-dictionaries-in-python.html
+def dict_merge(a, b):
+    '''recursively merges dict's. not just simple a['key'] = b['key'], if
+    both a and bhave a key who's value is a dict then dict_merge is called
+    on both values and the result stored in the returned dictionary.'''
+    if not isinstance(b, dict):
+        return b
+    result = copy.deepcopy(a)
+    for k, v in b.iteritems():
+        if k in result and isinstance(result[k], dict):
+            result[k] = dict_merge(result[k], v)
+        else:
+            result[k] = copy.deepcopy(v)
+    return result
+
+
 class ErrorIgnoringJSONEncoder(json.JSONEncoder):
     def __init__(self, **kw):
         kw.setdefault('skipkeys', True)
